package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

public class ExampleMain {

    public static void main(String[] args) {
        EntityManagerFactory emf =
                Persistence.createEntityManagerFactory("example-unit");
        try {

            int id = persistEntity(emf);
            runNativeQueries(emf);
            removeEntity(emf, id);
            runNativeQueries(emf);

        } finally {
            emf.close();
        }
    }

    private static void removeEntity(EntityManagerFactory emf, int id) {
        EntityManager em = emf.createEntityManager();
        Employee employee = em.find(Employee.class, id);
        System.out.println("-- removing entity --");
        printObject(employee);

        em.getTransaction().begin();
        em.remove(employee);
        em.getTransaction().commit();
        em.close();
    }


    private static int persistEntity(EntityManagerFactory emf) {
        System.out.println("-- Persisting Employee entity --");
        EntityManager em = emf.createEntityManager();
        Task task = new Task();
        task.setName("Refactoring");
        task.setStartDate(new Date());

        Employee employee = new Employee();
        employee.setName("Maria Dorsey");
        employee.setTasks(Arrays.asList(task));

        em.getTransaction().begin();
        em.persist(employee);
        em.getTransaction().commit();
        em.close();
        return employee.getId();
    }

    private static void runNativeQueries(EntityManagerFactory emf) {
        System.out.println("-- Native queries --");
        EntityManager em = emf.createEntityManager();
        ExampleMain.nativeQuery(em, "Select * from Employee");
        ExampleMain.nativeQuery(em, "Select * from Task");
        em.close();
    }

    public static void nativeQuery(EntityManager em, String s) {
        System.out.printf("'%s'%n", s);
        Query query = em.createNativeQuery(s);
        List list = query.getResultList();
        for (Object o : list) {
            if (o instanceof Object[]) {
                System.out.println(Arrays.toString((Object[]) o));
            } else {
                System.out.println(o);
            }
        }
    }

    private static void printObject(Object obj) {
        System.out.println("Object: " + obj);
    }
}